"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var readlineSync = require("readline-sync");
var date_fns_1 = require("date-fns");
var JiraStore_1 = require("./JiraStore");
var JiraClient_1 = require("./JiraClient");
var util_1 = require("./util");
var Logger_1 = require("../util/Logger");
var JiraApiConnector = /** @class */ (function () {
    function JiraApiConnector(saveCredentials, syncAll) {
        var _this = this;
        if (saveCredentials === void 0) { saveCredentials = false; }
        if (syncAll === void 0) { syncAll = false; }
        this.syncAll = syncAll;
        this.store = new JiraStore_1.JiraStore();
        this.transformEntry = function (entry, date) {
            var dateWithHours = date_fns_1.setHours(date, entry.startTime.getHours());
            var dateWithMinutes = date_fns_1.setMinutes(dateWithHours, entry.startTime.getMinutes());
            return {
                comment: entry.description,
                issue: {
                    key: entry.ticketId
                },
                author: {
                    name: _this.credentials.username
                },
                timeSpentSeconds: date_fns_1.differenceInSeconds(entry.endTime, entry.startTime),
                dateStarted: dateWithMinutes
            };
        };
        var host = this.store.getHost();
        if (!host) {
            host = JiraApiConnector.updateHost();
        }
        var credentials = this.store.getCredentials();
        if (!credentials) {
            credentials = JiraApiConnector.updateCredentials(saveCredentials);
        }
        this.credentials = credentials;
        this.client = new JiraClient_1.JiraClient(host, this.credentials);
    }
    JiraApiConnector.updateHost = function () {
        console.log("Please enter the host of your Jira instance \n");
        var host = readlineSync.question("Jira-Host: ");
        new JiraStore_1.JiraStore().setHost(host);
        return host;
    };
    JiraApiConnector.updateCredentials = function (saveCredentials) {
        console.log("Please enter you Jira credentials");
        console.log("");
        var username = readlineSync.question("Username: ");
        var password = readlineSync.question("Password: ", {
            hideEchoBack: true
        });
        var credentials = {
            username: username,
            password: password
        };
        if (saveCredentials) {
            Logger_1.Logger.info("Saving credentials");
            new JiraStore_1.JiraStore().setCredentials(credentials);
        }
        return credentials;
    };
    JiraApiConnector.prototype.importLogs = function (days) {
        return __awaiter(this, void 0, void 0, function () {
            var preparedEntries, oldEntries;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        Logger_1.Logger.withStep("Obtaining Cookie...", 1, 4);
                        return [4 /*yield*/, this.client.obtainCookie()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.prepareEntries(days)];
                    case 2:
                        preparedEntries = _a.sent();
                        oldEntries = this.store.getExistingEntries();
                        return [4 /*yield*/, this.createWorklogs(preparedEntries)];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, this.clearOldEntries(oldEntries, preparedEntries)];
                    case 4:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    JiraApiConnector.prototype.prepareEntries = function (days) {
        return __awaiter(this, void 0, void 0, function () {
            var startDate, _a;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        Logger_1.Logger.withStep("Fetching relevant date range...", 2, 4);
                        if (!this.syncAll) return [3 /*break*/, 1];
                        _a = null;
                        return [3 /*break*/, 3];
                    case 1: return [4 /*yield*/, this.client.getStartDateOfCurrentApprovalPeriod()];
                    case 2:
                        _a = _b.sent();
                        _b.label = 3;
                    case 3:
                        startDate = _a;
                        return [2 /*return*/, days
                                .filter(function (day) {
                                return _this.syncAll ||
                                    date_fns_1.isSameDay(day.date, startDate) ||
                                    date_fns_1.isAfter(day.date, startDate);
                            })
                                .map(function (_a) {
                                var date = _a.date, workEntries = _a.workEntries;
                                return workEntries.map(function (entry) { return _this.transformEntry(entry, date); });
                            })
                                .reduce(function (arr, curr) { return curr.concat(arr); }, [])];
                }
            });
        });
    };
    JiraApiConnector.prototype.createWorklogs = function (inputs) {
        return __awaiter(this, void 0, void 0, function () {
            var createdLogs, logsWithIds;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        Logger_1.Logger.withStep("Creating Worklogs...", 3, 4);
                        return [4 /*yield*/, util_1.executeTasks(inputs.map(function (log) { return function () { return _this.client.createWorklog(log); }; }))];
                    case 1:
                        createdLogs = _a.sent();
                        logsWithIds = createdLogs.filter(function (log) { return !!log; });
                        this.store.setExistingEntries(logsWithIds);
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Clears logs that were previously written and are part of the current batch
     * @param logs
     * @param onDays
     */
    JiraApiConnector.prototype.clearOldEntries = function (logs, onDays) {
        var _a;
        return __awaiter(this, void 0, void 0, function () {
            var relevantLogs;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        relevantLogs = (_a = logs) === null || _a === void 0 ? void 0 : _a.filter(function (entry) {
                            return onDays.some(function (day) { return date_fns_1.isSameDay(entry.date, day.dateStarted); });
                        });
                        Logger_1.Logger.withStep("Clearing old logs...", 4, 4);
                        if (!relevantLogs || !relevantLogs.length) {
                            Logger_1.Logger.info("No Logs to erase found");
                            return [2 /*return*/];
                        }
                        return [4 /*yield*/, util_1.executeTasks(relevantLogs.map(function (log) { return function () { return _this.client.deleteWorklog(log); }; }))];
                    case 1:
                        _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    return JiraApiConnector;
}());
exports.JiraApiConnector = JiraApiConnector;
